/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.nameTable;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Name table for mapping
 * @author ib27688
 */
public class NameToAddressMap {
	
	/**
	 * Mapping from strings to numbers.
	 */
	protected Map<String, Integer> nameToAddressMap;
	
	public NameToAddressMap() {
		nameToAddressMap= new HashMap<String, Integer>();
	}
		
	public Integer addName(String name, int address) {
		return nameToAddressMap.put(name, address);
	}
	
	/**
	 * Return the address associated with a name. Negative address means
	 * that no association has been established.
	 * 
	 * @param name 
	 * @return address associated with the name, a negative number if no
	 *         association exist
	 */
	public int getAddress(String name)
	throws NameUnknownException {
		Integer address= nameToAddressMap.get(name);
		if (address != null) {
			return address;
		}
		else {
			throw new NameUnknownException("Name " + name + "does not have a mapping to an address");
		}
	}
	
	public void clear() {
		nameToAddressMap.clear();
	}
	
	public String toString() {
		
		String mapString= "";
		
		for (Entry<String, Integer> e : nameToAddressMap.entrySet() ) {
			mapString= mapString
					+ "\n[ key = "
					+ e.getKey()
					+ ", value= "
					+ e.getValue()
					+ " ]";
		}
		
		return mapString;
	}
	
} // class NameTable
