/*
 * Created on Feb 2, 2006
 *
 */
package com.sunlabs.archsim.fleet.ships.fifo;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.State;


/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */


/**
 * @author ib27688
 *
 */

public class FifoShip extends Component {

	protected FifoShipElement first;
	protected FifoShipElement last;
	
	
	// to be used with XML front-end
	public FifoShip(String name, Parameters params) {
		super(name, null);
		
		int capacityParam= FifoDefaults.FIFO_CAPACITY_DEF;
		Delay fwdDelayParam= null;
		Delay ackDelayParam= null;
		
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				if (pName.equals(FifoDefaults.FIFO_CAPACITY) 
						|| pName.equals(FleetGlobal.PARAMETER) ) {
					try {
						capacityParam = Integer.parseInt(sValue);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " 
								+ FifoDefaults.FIFO_CAPACITY
								+ ": " + sValue
								+ ". Value is IGNORED");						
					}
				}
				else if (pName.equals(FifoDefaults.FIFO_SHIP_FWD_DELAY) 
						|| pName.equals(FifoDefaults.FWD_DELAY)) {
					try {
						int d = Integer.parseInt(sValue);
						fwdDelayParam= new Delay(d);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " 
								+ FifoDefaults.FIFO_SHIP_FWD_DELAY
								+ ": " + sValue
								+ ". Value is IGNORED");						
					}
				}
				else if (pName.equals(FifoDefaults.FIFO_SHIP_ACK_DELAY) 
						|| pName.equals(FifoDefaults.ACK_DELAY)) {
					try {
						int d = Integer.parseInt(sValue);
						ackDelayParam= new Delay(d);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " 
								+ FifoDefaults.FIFO_SHIP_ACK_DELAY
								+ ": " + sValue
								+ ". Value is IGNORED");						
					}
				}
				else {
					director.error("Component: " + name + ", id= " + getID() 
							+ ": Attribute " + pName + " with value " + sValue
							+ " is ignored");					
				} // else
				
			} // for	
		} // if		
		
		buildFifoShip(capacityParam, fwdDelayParam, ackDelayParam);
		
	}
	
	// to be used with XML front-end
	public FifoShip(String name, int capacity, Parameters params) {
		super(name, null);
		
		int capacityParam= capacity;
		Delay fwdDelayParam= null;
		Delay ackDelayParam= null;
		
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
				if (pName.equals(FifoDefaults.FIFO_CAPACITY) 
						|| pName.equals(FleetGlobal.PARAMETER) ) {
					// ignore capacity
				}
				else if (pName.equals(FifoDefaults.FIFO_SHIP_FWD_DELAY) 
						|| pName.equals(FifoDefaults.FWD_DELAY)) {
					try {
						int d = Integer.parseInt(sValue);
						fwdDelayParam= new Delay(d);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " 
								+ FifoDefaults.FIFO_SHIP_FWD_DELAY
								+ ": " + sValue
								+ ". Value is IGNORED");						
					}
				}
				else if (pName.equals(FifoDefaults.FIFO_SHIP_ACK_DELAY) 
						|| pName.equals(FifoDefaults.ACK_DELAY)) {
					try {
						int d = Integer.parseInt(sValue);
						ackDelayParam= new Delay(d);
					}
					catch (Exception e) {
						director.error("Component: " + name + ", id= " + getID() 
								+ ". Bogus value used for " 
								+ FifoDefaults.FIFO_SHIP_ACK_DELAY
								+ ": " + sValue
								+ ". Value is IGNORED");						
					}
				}
				else {
					director.error("Component: " + name + ", id= " + getID() 
							+ ": Attribute " + pName + " with value " + sValue
							+ " is ignored");					
				} // else
				
			} // for	
		} // if		
		
		buildFifoShip(capacityParam, fwdDelayParam, ackDelayParam);
		
	}
	
	public FifoShip(String name, int capacity) {
		super(name, null);
		buildFifoShip(capacity, null, null);		
	}
	
	protected void buildFifoShip(int capacity, Delay fwdDelay, Delay ackDelay) {
		// no negative capacity - make a zero capacity fifo
		if (capacity < 0) capacity= 0;
		
		// make the first element 
		first= new FifoShipElement();
		last= first; // need this for capacity 0 
		addMember(first);
		
		for (int i=0; i< capacity; i++) {
			// make the next F
			FifoShipElement next=  new FifoShipElement();
			addMember(next);
			
			// set delays if needed
			if (fwdDelay != null) {
				next.action.passDataFn.setDelay(fwdDelay);
			}
			if (ackDelay != null) {
				next.action.ackFn.setDelay(ackDelay);
			}			
			
			// make states and connect them to terminals
			State dataState= new State(FleetGlobal.DATA_STATE);
			addMember(dataState);
			dataState.attach(last.out);
			dataState.attach(next.in);
			
			// ackState must b initially active
			State ackState= new State(FleetGlobal.ACK_STATE);
			addMember(ackState);
			ackState.attach(last.ackOut);
			ackState.attach(next.ackIn);
			ackState.setInitiallyActive(new Data());
			
			// the ack state must be active
			// ack state was set active at creation
			// ackState.setActive(null);
			
			// prepare for the next iteration
			last= next;
		}		
	}
	
	public Terminal getTerminal(String name) {
		if (name.equals(FleetGlobal.IN)) {
			return first.in;
		}
		else if (name.equals(FleetGlobal.ACK_IN)) {
			return first.ackIn;
		}
		else if (name.equals(FleetGlobal.OUT)) {
			return last.out;
		}
		else if (name.equals(FleetGlobal.ACK_OUT)) {
			return last.ackOut;
		}
		else
			return null;		
	}

} // Fifo
