/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Feb 10, 2005
 *
 */
package com.sunlabs.archsim.tests.arbiter;


import com.sunlabs.archsim.gasp.State;

/** Generate the structure needed for the arbiter test: two sources,
 * connected to two actions with arbiter ports, connected to one sink.
 * That is, the test structure is a merge of two fifo's.
 * 
 * @author ib27688
 *
 */
public class ArbiterTestFactory {

	private ArbiterTestFactory() {}
	
	public static ArbiterTest please(String name) {
		
		ArbiterTest test= new ArbiterTest(name);
		
		// make the source A
		ArbiterTestSource srcA= new ArbiterTestSource(
				ArbiterTestGlobal.SOURCE_ACTION_A_NAME, 0);
		srcA.setParent(test);
		
		// make the terminals that attach to the source
		ArbiterTestOutput dataOutA= new ArbiterTestOutput(
				ArbiterTestGlobal.DATA_OUT_TERMINAL_NAME);
		dataOutA.setParent(test);
		
		ArbiterTestInput ackDataOutA= new ArbiterTestInput(
				ArbiterTestGlobal.ACK_DATA_OUT_TERMINAL_NAME);
		ackDataOutA.setParent(test);
		
		// attache the terminals to the source action
		srcA.attach(dataOutA);
		srcA.attach(ackDataOutA);
		
		// now make the states between the source A and
		// the action AC
		State dataA= new State(ArbiterTestGlobal.DATA_IN_A_STATE_NAME);
		dataA.setParent(test);
		
		State ackDataA= new State(
				ArbiterTestGlobal.ACK_DATA_IN_A_STATE_NAME);
		ackDataA.setParent(test);
		
		// attach terminals to these states
		dataA.attach(dataOutA);
		ackDataA.attach(ackDataOutA);
		
		//-----------------
		// make action from source A to sink (C)...
		ArbiterTestAction actionAC= new ArbiterTestAction(
				ArbiterTestGlobal.ACTION__AC_NAME);
		actionAC.setParent(test);
		
		// make terminals on the data source side of action AC
		ArbiterTestArbiterInput dataInAC= new ArbiterTestArbiterInput(
				ArbiterTestGlobal.DATA_IN_TERMINAL_NAME);
		// ArbiterTestInput dataInAC= new ArbiterTestInput(
		//				ArbiterTestGlobal.DATA_IN_TERMINAL_NAME);
		dataInAC.setParent(test);
		
		ArbiterTestOutput ackDataInAC= new ArbiterTestOutput(
				ArbiterTestGlobal.ACK_DATA_IN_TERMINAL_NAME);
		ackDataInAC.setParent(test);
		
		// attach the terminals to the action
		actionAC.attach(dataInAC);
		actionAC.attach(ackDataInAC);
		
		// attach the terminals to the states
		dataA.attach(dataInAC);
		ackDataA.attach(ackDataInAC);
		
		//------------------
		// make the source B
		ArbiterTestSource srcB= new ArbiterTestSource(
				ArbiterTestGlobal.SOURCE_ACTION_B_NAME, 1000);
		srcB.setParent(test);
		
		// make the terminals that attach to the source
		ArbiterTestOutput dataOutB= new ArbiterTestOutput(
				ArbiterTestGlobal.DATA_OUT_TERMINAL_NAME);
		dataOutB.setParent(test);
		
		ArbiterTestInput ackDataOutB= new ArbiterTestInput(
				ArbiterTestGlobal.ACK_DATA_OUT_TERMINAL_NAME);
		ackDataOutB.setParent(test);
		
		// attache the terminals to the source action
		srcB.attach(dataOutB);
		srcB.attach(ackDataOutB);
		
		// now make the states between the source B and
		// the action BC
		State dataB= new State(ArbiterTestGlobal.DATA_IN_B_STATE_NAME);
		dataB.setParent(test);
		
		State ackDataB= new State(
				ArbiterTestGlobal.ACK_DATA_IN_B_STATE_NAME);
		ackDataB.setParent(test);
		
		// attach terminals to these stated
		dataB.attach(dataOutB);
		ackDataB.attach(ackDataOutB);
		
		//-------------------
		// make action from source B to sink (C)...
		ArbiterTestAction actionBC= new ArbiterTestAction(
				ArbiterTestGlobal.ACTION_BC_NAME);
		actionBC.setParent(test);
		
		// make terminals on the data source side of action BC
		ArbiterTestArbiterInput dataInBC= new ArbiterTestArbiterInput(
				ArbiterTestGlobal.DATA_IN_TERMINAL_NAME);
		dataInBC.setParent(test);
		
		ArbiterTestOutput ackDataInBC= new ArbiterTestOutput(
				ArbiterTestGlobal.ACK_DATA_IN_TERMINAL_NAME);
		ackDataInBC.setParent(test);
		
		// attach the terminals to the action
		actionBC.attach(dataInBC);
		actionBC.attach(ackDataInBC);
		
		// attach the terminals to the states
		dataB.attach(dataInBC);
		ackDataB.attach(ackDataInBC);
		
		// connect the two arbitration terminals. that is, make them
		// peers so that only one at a time can report that it's
		// attached state is active
		/**
		 * TODO put this back in
		 */
		dataInAC.attachPeer(dataInBC);
		
		// now make the terminals on the sink end of action AC
		dataOutA= new ArbiterTestOutput(
				ArbiterTestGlobal.DATA_OUT_TERMINAL_NAME);
		dataOutA.setParent(test);
		
		ackDataOutA= new ArbiterTestInput(
				ArbiterTestGlobal.ACK_DATA_OUT_TERMINAL_NAME);
		ackDataOutA.setParent(test);
		
		// attach the terminals to the action
		actionAC.attach(dataOutA);
		actionAC.attach(ackDataOutA);
		
		// make the states that attach to these terminals
		State dataC= new State(
				ArbiterTestGlobal.DATA_OUT_C_STATE_NAME);
		dataC.setParent(test);
		
		State ackDataC= new State(
				ArbiterTestGlobal.ACK_DATA_OUT_C_STATE_NAME);
		ackDataC.setParent(test);
		
		// attach terminals to the states
		dataC.attach(dataOutA);
		ackDataC.attach(ackDataOutA);
		
		// now make the terminals on the sink end of action BC
		dataOutB= new ArbiterTestOutput(
				ArbiterTestGlobal.DATA_OUT_TERMINAL_NAME);
		dataOutB.setParent(test);
		
		ackDataOutB= new ArbiterTestInput(
				ArbiterTestGlobal.ACK_DATA_OUT_TERMINAL_NAME);
		ackDataOutB.setParent(test);
		
		// attach the terminals to the action
		actionBC.attach(dataOutB);
		actionBC.attach(ackDataOutB);
		
		// attach terminals to the states
		dataC.attach(dataOutB);
		ackDataC.attach(ackDataOutB);
		
		// make the sink
		ArbiterTestSink sink= new ArbiterTestSink(ArbiterTestGlobal.SINK_ACTION_NAME);
		sink.setParent(test);
		
		// make the terminals that attach to the sink
		ArbiterTestInput dataIn= new ArbiterTestInput(
				ArbiterTestGlobal.DATA_IN_TERMINAL_NAME);
		dataIn.setParent(test);
		
		ArbiterTestOutput ackDataIn= new ArbiterTestOutput(
				ArbiterTestGlobal.ACK_DATA_IN_TERMINAL_NAME);
		ackDataIn.setParent(test);
		
		// attach the terminals to the sink
		sink.attach(dataIn);
		sink.attach(ackDataIn);
		
		// attach the terminals to the states
		dataC.attach(dataIn);
		ackDataC.attach(ackDataIn);
		
		// activate the ack-states to enable the simulation
		ackDataA.setActive(null);
		ackDataB.setActive(null);
		ackDataC.setActive(null);
		
		return test;
	} // please
	
	
} // class ArbiterTestFactory
